@@ -2,6 +2,7 @@
 #include "fingerprint.h"
 #include "files.h"
 
+
 void HandleCheckFail(char *Path, char *ErrorMessage)
 {
 char *Tempstr=NULL;
@@ -9,12 +10,7 @@ char *Tempstr=NULL;
   if (Flags & FLAG_COLOR) printf("%s%s: FAILED. '%s'.%s\n",ANSICode(ANSI_RED, 0, 0),Path, ErrorMessage, ANSI_NORM);
 	else printf("%s: FAILED. %s.\n",Path,ErrorMessage);
 
-  if (StrLen(DiffHook))
-  {
-    Tempstr=MCopyStr(Tempstr,DiffHook," '",Path,"'",NULL);
-    system(Tempstr);
-  }
-
+	RunHookScript(DiffHook, Path, "");
   DestroyString(Tempstr);
 }
 
@@ -71,6 +67,7 @@ int CheckFileHash(HashratCtx *Ctx, char *Path, struct stat *Stat, char *ActualHa
 int result=FALSE;
 
 	if (strcasecmp(FP->Hash,ActualHash) != 0) HandleCheckFail(Path, "Hash mismatch");
+	else
 	{
 		if (! (Flags & FLAG_OUTPUT_FAILS))
 		{
@@ -118,6 +115,7 @@ return(result);
 }
 
 
+//returns true on a significant event, meaning on FAIL
 int CheckHashesFromList(HashratCtx *Ctx)
 {
 char *HashStr=NULL, *ptr;
@@ -133,7 +131,7 @@ if (strcmp(ptr,"-")==0)
   ListStream=STREAMFromFD(0);
   STREAMSetTimeout(ListStream,0);
 }
-else ListStream=STREAMOpenFile(ptr, STREAM_RDONLY);
+else ListStream=STREAMOpenFile(ptr, SF_RDONLY);
 
 FP=FingerprintRead(ListStream);
 while (FP)
@@ -150,7 +148,8 @@ while (FP)
 fprintf(stderr,"\nChecked %d files. %d Failures\n",Checked,Errors);
 
 DestroyString(HashStr);
-if (Errors) return(FALSE);
-return(TRUE);
+
+if (Errors) return(TRUE);
+return(FALSE);
 }
 
