@@ -72,6 +72,45 @@ fi
 }
 
 
+TestLocateHook()
+{
+rm -f locatehook.out
+HR_OUT=`echo $1 | ./hashrat -m -r . -hook "echo found > locatehook.out"`
+
+if [ -e locatehook.out ]
+then
+	OkayMessage "$3 works."
+else 
+	FailMessage "$3 BROKEN."
+fi
+}
+
+
+
+TestExitCodes()
+{
+if [ "$4" = "FindDuplicates" ]
+then
+	HR_OUT=`./hashrat -r -dups $1`
+	EXIT_FOUND=$?
+	HR_OUT=`./hashrat -r -dups $2`
+	EXIT_NOTFOUND=$?
+else
+	HR_OUT=`echo $1 $2 | ./hashrat $3 2>/dev/null`
+	EXIT_FOUND=$?
+	HR_OUT=`echo $1x $2 | ./hashrat $3 2>/dev/null`
+	EXIT_NOTFOUND=$?
+fi
+
+if [ "$EXIT_FOUND" = "0" -a "$EXIT_NOTFOUND" = "1" ]
+then
+	OkayMessage "$4 exit codes correct"
+else 
+	FailMessage "$4 exit codes BROKEN."
+fi
+}
+
+
 
 ##################### MAIN STARTS HERE ##########################
 
@@ -114,7 +153,7 @@ TestHash z85 "ZEROMQ85 encoding" "wX%ElWFTQ9+Z=X4h"
 Title "Testing Misc. Features"
 
 HR_OUT=`./hashrat -version`
-if [ "$HR_OUT" = "version: 1.8" ]
+if [ "$HR_OUT" = "version: 1.8.10" ]
 then
 	OkayMessage "Version (-version) works"
 else
@@ -130,8 +169,17 @@ else
 	FailMessage "File hashing BROKEN"
 fi
 
+HR_OUT=`./hashrat -dir -sha1 -trad tests`
+if [ "$HR_OUT" = "9521674698e62496698c42f63c9cde9bc6399a03  tests" ]
+then
+  OkayMessage "Directory hashing works"
+else
+  FailMessage "Directory hashing BROKEN"
+fi
+
+
 HR_OUT=`./hashrat -sha1 -trad -r tests | ./hashrat -sha1`
-if [ "$HR_OUT" = "06af1d9f777bbeb1eecd76d71d869089683ded1b" ]
+if [ "$HR_OUT" = "a31f3b27b58f8b26a3c2237beb9de410d1117fba" ]
 then
 	OkayMessage "Recursive file hashing works"
 else
@@ -148,7 +196,7 @@ else
 	FailMessage "Checking files BROKEN"
 fi
 
-HR_OUT=`./hashrat -r -dups tests`                      
+HR_OUT=`./hashrat -r -dups tests`
 if [ "$HR_OUT" = "DUPLICATE: tests/quotes.txt of tests/duplicate.txt " ]
 then
 	OkayMessage "Finding duplicate files works"
@@ -166,5 +214,14 @@ TestLocate "MD5 (test file) = 6ec9de513a8ff1768eb4768236198cf3" "LOCATED: 6ec9de
 HR_INPUT=`cat tests/test.ioc`
 TestLocate "$HR_INPUT" "LOCATED: 6ec9de513a8ff1768eb4768236198cf3 ' Hashrat Test IOC' at ./tests/help.txt" "Locating files with OpenIOC input"
 
+Title "Test hook functions"
+TestLocateHook "hash='md5:6ec9de513a8ff1768eb4768236198cf3' mode='100644' uid='0' gid='0' size='621' mtime='1423180289' inode='2359456' path='test file'" "" "Hook function for file locate"
+TestLocateHook "hash='md5:6933ee7eb504d29312b23a47d2dac374' mode='100644' uid='0' gid='0' size='621' mtime='1423180289' inode='2359456' path='test file'" "" "Hook function for file locate of files with bad characters in name"
+
+Title "Testing exit codes for different operations"
+
+TestExitCodes "6ec9de513a8ff1768eb4768236198cf3" "tests/help.txt" "-cf" "CheckHash"
+TestExitCodes "6ec9de513a8ff1768eb4768236198cf3" "tests/help.txt" "-m -r ." "Locate"
+TestExitCodes "tests" "libUseful-2.5" "-r -dups" "FindDuplicates"
 
 exit $EXIT
