@@ -2,12 +2,13 @@
 #ifndef HASHRAT_COMMON_H
 #define HASHRAT_COMMON_H
 
-#include "libUseful-2.1/libUseful.h"
+#include "libUseful-2.5/libUseful.h"
 #include "glob.h"
 
 
 #define ACT_NONE 0
 #define ACT_HASH 1
+#define ACT_HASHDIR 2
 #define ACT_HASH_LISTFILE 3
 #define ACT_PRINTUSAGE 4
 #define ACT_CGI 6
@@ -28,7 +29,6 @@
 //Two flags with the same values, but used in different contexts
 #define FLAG_ERROR 2
 #define FLAG_VERBOSE 2
-#define FLAG_DIRMODE 4
 #define FLAG_DEVMODE 8
 #define FLAG_DIR_INFO 32
 #define FLAG_TRAD_OUTPUT 64
@@ -59,11 +59,6 @@
 #define CTX_INCLUDE 16384
 #define CTX_EXCLUDE 32768
 
-#define INEX_INCLUDE 1
-#define INEX_EXCLUDE 2
-#define INEX_INCLUDE_DIR 3
-#define INEX_EXCLUDE_DIR 4
-
 #define RESULT_PASS 1
 #define RESULT_FAIL 2
 #define RESULT_WARN 4
@@ -75,7 +70,9 @@
 
 #define BLOCKSIZE 4096
 
-#define VERSION "1.8.1"
+#define IGNORE -1
+
+#define VERSION "1.8.11"
 
 
 typedef struct
@@ -118,6 +115,6 @@ TFingerprint *TFingerprintCreate(const char *Hash, const char *HashType, const c
 void HashratCtxDestroy(void *p_Ctx);
 void HashratStoreHash(HashratCtx *Ctx, char *Path, struct stat *Stat, char *Hash);
 int HashratOutputInfo(HashratCtx *Ctx, STREAM *S, char *Path, struct stat *Stat, char *Hash);
-void HandleCompareResult(char *Path, char *Status, int Flags, char *ErrorMessage);
+void RunHookScript(const char *Hook, const char *Path, const char *Other);
 
 #endif
