@@ -52,7 +52,7 @@ char *mptr, *dptr;
 int result=TRUE;
 
 if (Ctx->Flags & CTX_EXCLUDE) result=FALSE;
-if (S_ISDIR(FStat->st_mode)) result=TRUE;
+if (FStat && S_ISDIR(FStat->st_mode)) result=TRUE;
 
 Curr=ListGetNext(IncludeExclude);
 while (Curr)
@@ -60,35 +60,48 @@ while (Curr)
 	mptr=(char *) Curr->Item;
 	dptr=Path;
 
-	if (*mptr!='/') 
+	//if match pattern doesn't start with '/' then we want to strip that off the current path
+	//so that we can match against it. However if the match pattern contains no '/' at all, then
+	//it's a file name rather than a path, in which case we should use basename on both it and 
+	//the current path
+	if (*mptr != '/') 
 	{
+		if (strchr(mptr,'/'))
+		{
+			if (*dptr=='/') dptr++;
+		}
+		else
+		{
 		mptr=GetBasename(mptr);
 		dptr=GetBasename(Path);
+		}
 	}
 	
 	switch (Curr->ItemType)
 	{
-	case INEX_INCLUDE:
+	case CTX_INCLUDE:
 	if (fnmatch(mptr,dptr,0)==0) result=TRUE;
 	break;
 
-	case INEX_EXCLUDE:
+	case CTX_EXCLUDE:
 	if (fnmatch(mptr,dptr,0)==0) result=FALSE;
 	break;
 
+/*
 	case INEX_INCLUDE_DIR:
 	if (strncmp(mptr,dptr,StrLen(mptr))==0) result=TRUE;
 	break;
 
 	case INEX_EXCLUDE_DIR:
 	if (strncmp(mptr,dptr,StrLen(mptr))==0) result=FALSE;
+	printf("FNMD: [%s] [%s] %d\n",mptr,dptr,result);
 	break;
+*/
 	}
 
 	Curr=ListGetNext(Curr);
 }
 
-
 return(result);
 }
 
@@ -169,7 +182,7 @@ int HashratHashFile(HashratCtx *Ctx, THash *Hash, int Type, char *Path, off_t Fi
 STREAM *S;
 char *Tempstr=NULL, *User=NULL, *Pass=NULL;
 int result, val, RetVal=FALSE;
-off_t bytes_read=0;
+off_t bytes_read=0, oval;
 
 switch (Type)
 {
@@ -184,11 +197,11 @@ switch (Type)
 	break;
 
 	default:
-		if ((! StrLen(Path)) || (strcmp(Path,"-")==0)) S=STREAMFromFD(0);
+		if ((! StrValid(Path)) || (strcmp(Path,"-")==0)) S=STREAMFromFD(0);
 		else
 		{
-			if (Ctx->Flags & CTX_DEREFERENCE) S=STREAMOpenFile(Path,STREAM_RDONLY|STREAM_SYMLINK_OK);
-			else S=STREAMOpenFile(Path,STREAM_RDONLY);
+			if (Ctx->Flags & CTX_DEREFERENCE) S=STREAMOpenFile(Path,SF_RDONLY|SF_SYMLINK_OK);
+			else S=STREAMOpenFile(Path,SF_RDONLY);
 		}
 	break;
 }
@@ -197,8 +210,9 @@ if (S)
 {
 Tempstr=SetStrLen(Tempstr,BUFSIZ);
 
-val=FileSize;
-if ((val==0) || ( val > BUFSIZ)) val=BUFSIZ;
+oval=FileSize;
+if ((oval==0) || ( oval > BUFSIZ)) val=BUFSIZ;
+else val=(int)oval;
 result=STREAMReadBytes(S,Tempstr,val);
 while (result > 0)
 {
@@ -210,8 +224,9 @@ while (result > 0)
 	if (FileSize > 0)
 	{
 	if ((Type != FT_HTTP) && (bytes_read >= FileSize)) break;
-	val=FileSize - bytes_read;
-	if (val > BUFSIZ) val=BUFSIZ;
+	oval=FileSize - bytes_read;
+	if (oval > BUFSIZ) val=BUFSIZ;
+	else val=(int)oval;
 	}
 	result=STREAMReadBytes(S,Tempstr,val);
 }
@@ -237,15 +252,14 @@ void HashratFinishHash(char **RetStr, HashratCtx *Ctx, THash *Hash)
 int val;
 char *ptr;
 
-		Hash->Finish(Hash,Ctx->Encoding,RetStr);
+		HashFinish(Hash,Ctx->Encoding,RetStr);
 
 		ptr=GetVar(Ctx->Vars,"Output:Length");
-		if (StrLen(ptr))
+		if (StrValid(ptr))
 		{
 			val=atoi(ptr);
 			if ((val > 0) && (StrLen(*RetStr) > val)) (*RetStr)[val]='\0';
 		}
-		HashDestroy(Hash);
 }
 
 
@@ -255,7 +269,7 @@ int HashratHashSingleFile(HashratCtx *Ctx, char *HashType, int FileType, char *P
 THash *Hash;
 struct stat XattrStat;
 char *ptr;
-int size=0;
+off_t size=0;
 
 		*RetStr=CopyStr(*RetStr,"");
 
@@ -268,7 +282,7 @@ int size=0;
 		}
 
 
-		if (StrLen(*RetStr)==0)
+		if (! StrValid(*RetStr))
 		#endif
 		{
 			Hash=HashInit(HashType);
@@ -297,11 +311,14 @@ THash *Hash;
 		if (! Ctx->Hash) 
 		{
 			Hash=HashInit(Ctx->HashType);
+			if (Hash)
+			{
 			ptr=GetVar(Ctx->Vars,"EncryptionKey");
 			if (ptr) HMACSetKey(Hash, ptr, StrLen(ptr));
 
 			Hash->Update(Hash ,Data, DataLen);
 			HashratFinishHash(RetStr, Ctx, Hash);
+			}
 		} 
 		else Ctx->Hash->Update(Ctx->Hash ,Data, DataLen);
 }
@@ -312,6 +329,8 @@ int ConsiderItem(HashratCtx *Ctx, char *Path, struct stat *FStat)
 	int Type;
 
 	Type=FileType(Path, Flags, FStat);
+	if (! IsIncluded(Ctx, Path, FStat)) return(CTX_EXCLUDE);
+
 	switch (Type)
 	{
 		case FT_SSH:
@@ -331,7 +350,6 @@ int ConsiderItem(HashratCtx *Ctx, char *Path, struct stat *FStat)
 		else if (FStat->st_dev != StartingFS) return(CTX_ONE_FS);
 	}
 	if ((Ctx->Flags & CTX_EXES) && (! (FStat->st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)))) return(CTX_EXCLUDE);
-	if (! IsIncluded(Ctx,Path, FStat)) return(CTX_EXCLUDE);
 	}
 
 	return(0);
@@ -345,7 +363,6 @@ char *Tempstr=NULL;
 
 
 	Type=FileType(Path, Flags, FStat);
-
 	switch (Type)
 	{
 		case FT_HTTP:
@@ -401,12 +418,6 @@ char *Tempstr=NULL;
 		break;
 	}
 
-  if (StrLen(DiffHook))
-  {
-    Tempstr=MCopyStr(Tempstr,DiffHook," '",Path,"'",NULL);
-    system(Tempstr);
-  }
-
 
 DestroyString(Tempstr);
 
@@ -415,18 +426,28 @@ return(0);
 
 
 
-
-void HashratAction(HashratCtx *Ctx, char *Path, struct stat *Stat)
+//HashratAction returns true on a significant event, which is either an item found in search
+//or a check failing in hash-checking mode
+int HashratAction(HashratCtx *Ctx, char *Path, struct stat *Stat)
 {
 char *HashStr=NULL;
-TFingerprint *FP;
+int Type, result=FALSE;
+TFingerprint *FP=NULL;
 
 switch (Ctx->Action)
 {
+case ACT_HASHDIR:
+			Type=FileType(Path, Flags, Stat);
+			//we return TRUE if hash succeeded
+			if (HashratHashFile(Ctx, Ctx->Hash, Type, Path, Stat->st_size)) result=TRUE;
+break;
+
 case ACT_HASH:
 	HashItem(Ctx, Ctx->HashType, Path, Stat, &HashStr);
 	HashratOutputInfo(Ctx, Ctx->Out, Path, Stat, HashStr);
 	HashratStoreHash(Ctx, Path, Stat, HashStr);
+	//we return TRUE if hash succeeded
+	result=TRUE;
 break;
 
 case ACT_CHECK:
@@ -436,12 +457,13 @@ case ACT_CHECK:
 		{
 			HashItem(Ctx, Ctx->HashType, Path, Stat, &HashStr);
 			FP=CheckForMatch(Ctx, Path, Stat, HashStr);
-			if (FP)
+			if (FP && HashratCheckFile(Ctx, Path, Stat, HashStr, FP)) MatchCount++;
+			else 
 			{
-				if (HashratCheckFile(Ctx, Path, Stat, HashStr, FP)) MatchCount++;
+					HandleCheckFail(Path, "Changed or new");
+					//we return TRUE on FAILURE, as we are signaling a significant event
+					result=TRUE;
 			}
-			else HandleCheckFail(Path, "Changed or new");
-			TFingerprintDestroy(FP);	
 		}
 		else if (Flags & FLAG_VERBOSE) fprintf(stderr,"ZERO LENGTH FILE: %s\n",Path);
 	}
@@ -450,20 +472,27 @@ break;
 case ACT_CHECK_XATTR:
 	if (S_ISREG(Stat->st_mode))
 	{
+		//result == TRUE by default (TRUE==Signficant event, here meaning 'check failed')
+		result=TRUE;
 		FP=XAttrLoadHash(Ctx, Path);
 		if (FP) 
 		{
 			HashItem(Ctx, FP->HashType, Path, Stat, &HashStr);
-			if (FP->Flags & FP_HASSTAT) HashratCheckFile(Ctx, Path, Stat, HashStr, FP);
-			else HashratCheckFile(Ctx, Path, Stat, HashStr, FP);
+			if (FP->Flags & FP_HASSTAT) if (HashratCheckFile(Ctx, Path, Stat, HashStr, FP)) result=FALSE;
+			else if (HashratCheckFile(Ctx, Path, Stat, HashStr, FP)) result=FALSE;
 		}
 		else fprintf(stderr,"ERROR: No stored hash for '%s'\n",Path);
 	}
+	else fprintf(stderr,"ERROR: Not regular file '%s'. Not checking in xattr mode.\n",Path);
 break;
 
+
 case ACT_CHECK_MEMCACHED:
 	if (S_ISREG(Stat->st_mode))
 	{
+		//result == TRUE by default (TRUE==Signficant event, here meaning 'check failed')
+		result=TRUE;
+
 		if (Stat->st_size > 0)
 		{
 		HashItem(Ctx, Ctx->HashType, Path, Stat, &HashStr);
@@ -472,12 +501,12 @@ case ACT_CHECK_MEMCACHED:
 		else FP->Path=MCopyStr(FP->Path,"hashrat://",LocalHost,Path,NULL);
 		FP->Hash=MemcachedGet(FP->Hash, FP->Path);
 
-		if (FP) HashratCheckFile(Ctx, Path, NULL, HashStr, FP);
+		if (FP && HashratCheckFile(Ctx, Path, NULL, HashStr, FP)) result=FALSE;
 		else fprintf(stderr,"ERROR: No stored hash for '%s'\n",Path);
-		TFingerprintDestroy(FP);
 		}
 		else if (Flags & FLAG_VERBOSE) fprintf(stderr,"ZERO LENGTH FILE: %s\n",Path);
 	}
+	else fprintf(stderr,"ERROR: Not regular file '%s'. Not checking in memcached mode.\n",Path);
 break;
 
 case ACT_FINDMATCHES:
@@ -490,11 +519,13 @@ case ACT_FINDMATCHES_MEMCACHED:
 		FP=CheckForMatch(Ctx, Path, Stat, HashStr);
 		if (FP)
 		{
-			printf("LOCATED: %s '%s %s' at %s\n",FP->Hash,FP->Path,FP->Data,Path);
+			if (StrValid(FP->Path) || StrValid(FP->Data)) printf("LOCATED: %s '%s %s' at %s\n",FP->Hash, FP->Path, FP->Data, Path);
+			else printf("LOCATED: %s at %s\n",FP->Hash, Path);
 			MatchCount++;
+			//here we return true if a match found
+			result=TRUE;
 		}
 		else DiffCount++;
-		TFingerprintDestroy(FP);	
 		}
 		else if (Flags & FLAG_VERBOSE) fprintf(stderr,"ZERO LENGTH FILE: %s\n",Path);
 	}
@@ -512,13 +543,16 @@ case ACT_FINDDUPLICATES:
 			{
 				printf("DUPLICATE: %s of %s %s\n",Path,FP->Path,FP->Data);
 				MatchCount++;
-				TFingerprintDestroy(FP);	
+			//here we return true if a match found
+				result=TRUE;
 			}
 			else 
 			{
 				FP=TFingerprintCreate(HashStr, Ctx->HashType, "", Path);
 				DiffCount++;
 				MatchAdd(FP, Path, 0);
+				//as we've added FP to an internal list we don't want it destroyed
+				FP=NULL;
 			}
 		}
 	}
@@ -527,17 +561,27 @@ case ACT_FINDDUPLICATES:
 break;
 }
 
+if (result==TRUE) 
+{
+	if (FP) RunHookScript(DiffHook, Path, FP->Path);
+	else RunHookScript(DiffHook, Path, "");
+}
+
+if (FP) TFingerprintDestroy(FP);	
 DestroyString(HashStr);
+
+return(result);
 }
 
 
-
+//ProcessItem returns TRUE on a significant event, so any instance of TRUE
+//from items checked makes return value here TRUE
 int ProcessDir(HashratCtx *Ctx, char *Dir, char *HashType)
 {
 char *Tempstr=NULL, *HashStr=NULL;
 ListNode *FileList, *Curr;
+int result=FALSE;
 int Type;
-int result=TRUE;
 
 		Type=FileType(Dir, Flags, NULL);
 
@@ -547,7 +591,7 @@ int result=TRUE;
 		Curr=ListGetNext(FileList);
 		while (Curr)
 		{
-			ProcessItem(Ctx, Curr->Tag, (struct stat *) Curr->Item);
+			if (ProcessItem(Ctx, Curr->Tag, (struct stat *) Curr->Item)) result=TRUE;
 			Curr=ListGetNext(Curr);
 		}
 
@@ -560,23 +604,27 @@ return(result);
 }
 
 
-
-int HashratRecurse(HashratCtx *Ctx, char *Path, char **HashStr, int result)
+//ProcessDir returns TRUE on a significant event, so any instance of TRUE
+//from items checked makes return value here TRUE
+int HashratRecurse(HashratCtx *Ctx, char *Path, char **HashStr)
 {
 char *ptr;
-THash *Hash;
+struct stat FStat;
+int result=FALSE;
 
-
-		if ((Flags & FLAG_DIRMODE) && (! Ctx->Hash)) 
+		if ((Ctx->Action == ACT_HASHDIR) && (! Ctx->Hash)) 
 		{
-				Hash=HashInit(Ctx->HashType);
+				Ctx->Hash=HashInit(Ctx->HashType);
 				ptr=GetVar(Ctx->Vars,"EncryptionKey");
-				if (ptr) HMACSetKey(Hash, ptr, StrLen(ptr));
+				if (ptr) HMACSetKey(Ctx->Hash, ptr, StrLen(ptr));
 
 				if (! ProcessDir(Ctx, Path, Ctx->HashType)) result=FALSE;
-				HashratFinishHash(HashStr, Ctx, Hash);
+				HashratFinishHash(HashStr, Ctx, Ctx->Hash);
+				stat(Path, &FStat);
+				HashratOutputInfo(Ctx, Ctx->Out, Path, &FStat, *HashStr);
+				result=TRUE;
 		}
-		else if (! ProcessDir(Ctx, Path, Ctx->HashType)) result=FALSE;
+		else if (ProcessDir(Ctx, Path, Ctx->HashType)) result=TRUE;
 
 	return(result);
 }
@@ -584,26 +632,30 @@ THash *Hash;
 
 
 
-void ProcessItem(HashratCtx *Ctx, char *Path, struct stat *Stat)
+int ProcessItem(HashratCtx *Ctx, char *Path, struct stat *Stat)
 {
 char *HashStr=NULL;
+int result=FALSE, Flags;
 
 				switch (ConsiderItem(Ctx, Path, Stat))
 				{
 					case CTX_EXCLUDE:
 					case CTX_ONE_FS:
+						result=IGNORE;
 					break;
 
 					case CTX_RECURSE:
-					HashratRecurse(Ctx, Path, &HashStr, 0);
+					result=HashratRecurse(Ctx, Path, &HashStr);
 					break;
 
 					default:
-					HashratAction(Ctx, Path, Stat);
+					result=HashratAction(Ctx, Path, Stat);
 					break;
 				}
 
 DestroyString(HashStr);
+
+return(result);
 }
 
 
